@@ -9,13 +9,17 @@ COPYING distributed with the source files.
 #include <unistd.h>
 #include <limits.h>
 #include <signal.h>
+#include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
+#include <dirent.h>
 #include <fcntl.h>
 #include <errno.h>
+#include <pthread.h>
 
 #include <fstream>
 #include <iomanip>
+#include <memory>
 #include <ctime>
 #include <cstring>
 #include <cstdlib>
@@ -23,16 +27,13 @@ COPYING distributed with the source files.
 // uncomment for debugging in EfaxController::timer_event()
 //#include <iostream>
 
-#include <glibmm/main.h>
-#include <glibmm/convert.h>
-#include <glibmm/timer.h>
-#include <glibmm/thread.h>
+#include <glib/gtimer.h>
 
 #include "efax_controller.h"
-
-#ifdef HAVE_PTHREAD_SIGMASK 
-#include <pthread.h>
-#endif
+#include "thread.h"
+#include "mutex.h"
+#include "utf8_utils.h"
+#include "shared_handle.h"
 
 #ifdef HAVE_STRINGSTREAM
 #include <sstream>
@@ -44,15 +45,15 @@ COPYING distributed with the source files.
 #include <libintl.h>
 #endif
 
-EfaxController::EfaxController(void): child_pid(0), state(inactive),
+// the headers on one of my machines does not properly include the declaration
+// of getpgid() for some reason, so declare it here explicitly
+extern "C" pid_t getpgid(pid_t);
+
+
+EfaxController::EfaxController(void): state(inactive),
 				      close_down(false) {
   // set up state_messages
 
-  // it's OK to use std::string rather than Glib::ustring here, because
-  // the contents are passed to StatusLine::write_status(const char*) and
-  // std::string is just a transparent holder of the UTF-8 string
-  // until it reaches that method
-
   state_messages.push_back(gettext("Inactive"));
   state_messages.push_back(gettext("Sending fax"));
   state_messages.push_back(gettext("Answering call"));
@@ -61,6 +62,8 @@ EfaxController::EfaxController(void): child_pid(0), state(inactive),
   state_messages.push_back(gettext("Sending fax"));
   state_messages.push_back(gettext("Sending fax"));
 
+  cleanup_on_start();
+
   fax_made_notify.connect(sigc::mem_fun(*this, &EfaxController::sendfax_slot));
   no_fax_made_notify.connect(sigc::mem_fun(*this, &EfaxController::no_fax_made_slot));
 }
@@ -69,7 +72,7 @@ void EfaxController::efax_closedown(void) {
   if (state == inactive) ready_to_quit_notify();
   else if (!close_down) {
     close_down = true;
-    stop_slot();
+    stop();
   }
 }
 
@@ -94,11 +97,11 @@ void EfaxController::init_sendfax_parms(void) {
   temp += date_string;
   temp += "    ";
   try {
-    temp += Glib::locale_from_utf8(gettext("From"));
+    temp += Utf8::locale_from_utf8(gettext("From"));
     temp += " ";
-    temp += Glib::locale_from_utf8(prog_config.my_name);
+    temp += Utf8::locale_from_utf8(prog_config.my_name);
   }
-  catch (Glib::ConvertError&) {
+  catch (Utf8::ConversionError&) {
     write_error("UTF-8 conversion error in EfaxController::init_sendfax_parms()\n");
   }
   temp += " (";
@@ -136,7 +139,7 @@ void EfaxController::sendfax(const Fax_item& fax_item) {
   // want that test to occur on the call to this method in
   // EfaxController::timer_event()
   else if (state == inactive || state == start_send_on_standby) {
-    stdout_pipe.open(Pipe_fifo::non_block);
+    stdout_pipe.open(PipeFifo::non_block);
     // efax is sensitive to timing, so set pipe write to non-block also
     stdout_pipe.make_write_non_block();
     
@@ -176,27 +179,18 @@ void EfaxController::sendfax(const Fax_item& fax_item) {
     sigaddset(&sig_mask, SIGTERM);
     sigaddset(&sig_mask, SIGINT);
     sigaddset(&sig_mask, SIGHUP);
-    sigaddset(&sig_mask, SIGUSR2);
-#ifdef HAVE_PTHREAD_SIGMASK 
     pthread_sigmask(SIG_BLOCK, &sig_mask, 0);
-#else
-    while (sigprocmask(SIG_BLOCK, &sig_mask, 0) == -1 && errno == EINTR);
-#endif
-    try {
-      Glib::Thread::create(sigc::mem_fun(*this, &EfaxController::make_fax_thread), false);
-    }
-    catch (Glib::ThreadError&) {
+
+    std::auto_ptr<Thread::Thread> res =
+      Thread::Thread::start(sigc::mem_fun(*this, &EfaxController::make_fax_thread), false);
+    if (!res.get()) {
       write_error("Cannot start thread to make fax for sending\n");
       stdout_pipe.close();
       state = inactive;
       display_state();
     }
     // now unblock signals so that the initial (GUI) thread can receive them
-#ifdef HAVE_PTHREAD_SIGMASK 
     pthread_sigmask(SIG_UNBLOCK, &sig_mask, 0);
-#else
-    while (sigprocmask(SIG_UNBLOCK, &sig_mask, 0) == -1 && errno == EINTR);
-#endif
   }
   else beep();
 }
@@ -243,10 +237,10 @@ std::pair<const char*, char* const*> EfaxController::get_sendfax_parms(void) {
 void EfaxController::sendfax_slot(void) {
 
   // this method runs in the initial (GUI) thread -  we get this process
-  // to run in the initial (GUI) thread by invoking it from a Glib::Dispatcher
+  // to run in the initial (GUI) thread by invoking it from a Notifier
   // object in EfaxController::make_fax_thread(), which will execute the
   // slot attached to that object (this method) in the thread in which the
-  // Glib::Dispatcher object was created.  The Glib::Dispatcher object was
+  // Notifier object was created.  The Notifier object was
   // created as a member of the EfaxController object, and thus was created
   // in the initial (GUI) thread.  We need to execute this method in the GUI
   // thread because the program calls waitpid() in that thread to examine
@@ -265,17 +259,16 @@ void EfaxController::sendfax_slot(void) {
 
   // set up a synchronising pipe  in case the child process finishes before
   // fork() in parent space has returned (yes, with an exec() error that can
-  // happen with Linux kernel 2.6) - this is important because we test child_pid
-  // in efax_controller_childexit_handler()
-  Sync_pipe sync_pipe;
+  // happen with Linux kernel 2.6)
+  SyncPipe sync_pipe;
 
-  child_pid = fork();
+  prog_config.efax_controller_child_pid = fork();
 
-  if (child_pid == -1) {
+  if (prog_config.efax_controller_child_pid == -1) {
     write_error("Fork error - exiting\n");
     std::exit(FORK_ERROR);
   }
-  if (!child_pid) {  // child process - as soon as everything is set up we are going to do an exec()
+  if (!prog_config.efax_controller_child_pid) {  // child process - as soon as everything is set up we are going to do an exec()
 
     // now we have forked, we can connect stdout_pipe to stdout
     // and connect MainWindow::error_pipe to stderr
@@ -313,7 +306,7 @@ std::pair<const char*, char* const*> EfaxController::get_gs_parms(const std::str
 
   // lock the Prog_config object to stop it being modified in the intial (GUI) thread
   // while we are accessing it here
-  Glib::Mutex::Lock lock(*prog_config.mutex_p);
+  Thread::Mutex::Lock lock(*prog_config.mutex_p);
 
   std::vector<std::string> parms;
   std::string temp;
@@ -365,14 +358,14 @@ void EfaxController::make_fax_thread(void) {
 
     if (pos == std::string::npos || pos + 2 > filename_iter->size()) {
       try {
-	write_error(Glib::locale_from_utf8(gettext("Not valid file name\n")).c_str());
+	write_error(Utf8::locale_from_utf8(gettext("Not valid file name\n")).c_str());
       }
-      catch (Glib::ConvertError&) {
+      catch (Utf8::ConversionError&) {
 	write_error("UTF-8 conversion error in EfaxController::make_fax_thread()\n");
       }
 
       // synchronise memory on multi-processor systems before we emit the
-      // Glib::Dispatcher signal - we call fax_made_sem.wait() in
+      // Notifier signal - we call fax_made_sem.wait() in
       // EfaxController::no_fax_made_slot()
       fax_made_sem.post();
 
@@ -383,14 +376,14 @@ void EfaxController::make_fax_thread(void) {
     
     else if (access(filename_iter->c_str(), F_OK)) {
       try {
-	write_error(Glib::locale_from_utf8(gettext("File does not exist\n")).c_str());
+	write_error(Utf8::locale_from_utf8(gettext("File does not exist\n")).c_str());
       }
-      catch (Glib::ConvertError&) {
+      catch (Utf8::ConversionError&) {
 	write_error("UTF-8 conversion error in EfaxController::make_fax_thread()\n");
       }
 
       // synchronise memory on multi-processor systems before we emit the
-      // Glib::Dispatcher signal - we call fax_made_sem.wait() in
+      // Notifier signal - we call fax_made_sem.wait() in
       // EfaxController::no_fax_made_slot()
       fax_made_sem.post();
 
@@ -401,14 +394,14 @@ void EfaxController::make_fax_thread(void) {
 
     else if (access(filename_iter->c_str(), R_OK)) {
       try {
-	write_error(Glib::locale_from_utf8(gettext("User does not have read permission on the file\n")).c_str());
+	write_error(Utf8::locale_from_utf8(gettext("User does not have read permission on the file\n")).c_str());
       }
-      catch (Glib::ConvertError&) {
+      catch (Utf8::ConversionError&) {
 	write_error("UTF-8 conversion error in EfaxController::make_fax_thread()\n");
       }
 
       // synchronise memory on multi-processor systems before we emit the
-      // Glib::Dispatcher signal - we call fax_made_sem.wait() in
+      // Notifier signal - we call fax_made_sem.wait() in
       // EfaxController::no_fax_made_slot()
       fax_made_sem.post();
 
@@ -432,7 +425,7 @@ void EfaxController::make_fax_thread(void) {
     // create a synchronising pipe - we need to wait() on gs having completed executing
     // and then notify the parent process.  To wait() successfully we need to fork() once,
     // reset the child signal handler, and then fork() again
-    Sync_pipe sync_pipe;
+    SyncPipe sync_pipe;
 
     pid_t pid = fork();
     
@@ -452,7 +445,6 @@ void EfaxController::make_fax_thread(void) {
       sigaddset(&sig_mask, SIGTERM);
       sigaddset(&sig_mask, SIGINT);
       sigaddset(&sig_mask, SIGHUP);
-      sigaddset(&sig_mask, SIGUSR2);
       // this child process is single threaded, so we can use sigprocmask()
       // rather than pthread_sigmask() (and should do so as sigprocmask()
       // is guaranteed to be async-signal-safe)
@@ -577,14 +569,14 @@ void EfaxController::make_fax_thread(void) {
     
     if (!valid_file) {
       try {
-	write_error(Glib::locale_from_utf8(gettext("Not valid postscript file\n")).c_str());
+	write_error(Utf8::locale_from_utf8(gettext("Not valid postscript file\n")).c_str());
       }
-      catch (Glib::ConvertError&) {
+      catch (Utf8::ConversionError&) {
 	write_error("UTF-8 conversion error in EfaxController::make_fax_thread()\n");
       }
 
       // synchronise memory on multi-processor systems before we emit the
-      // Glib::Dispatcher signal - we call fax_made_sem.wait() in
+      // Notifier signal - we call fax_made_sem.wait() in
       // EfaxController::no_fax_made_slot()
       fax_made_sem.post();
 
@@ -598,7 +590,7 @@ void EfaxController::make_fax_thread(void) {
   // will cause sendfax_slot() to be executed in the initial (GUI) thread
 
   // synchronise memory on multi-processor systems before we emit the
-  // Glib::Dispatcher signal - we call fax_made_sem.wait() in
+  // Notifier signal - we call fax_made_sem.wait() in
   // EfaxController::sendfax_slot()
   fax_made_sem.post();
 
@@ -668,7 +660,7 @@ void EfaxController::receive(State mode) {
     const char format[] = "%y%m%d%H%M%S";
     {
       // lock the Prog_config object while we modify it
-      Glib::Mutex::Lock lock(*prog_config.mutex_p);
+      Thread::Mutex::Lock lock(*prog_config.mutex_p);
       std::strftime(prog_config.receive_dirname, MAX_TEMP_NAME + 1, format, time_p);
     }
     std::string temp(fax_pathname);
@@ -684,7 +676,7 @@ void EfaxController::receive(State mode) {
       time_p = std::localtime(&time_count);
       {
 	// lock the Prog_config object while we modify it
-	Glib::Mutex::Lock lock(*prog_config.mutex_p);
+	Thread::Mutex::Lock lock(*prog_config.mutex_p);
 	std::strftime(prog_config.receive_dirname, MAX_TEMP_NAME + 1, format, time_p);
       }
       temp = fax_pathname + prog_config.receive_dirname;
@@ -693,7 +685,7 @@ void EfaxController::receive(State mode) {
       write_error("Can't create directory to save fax\n");
       {
 	// lock the Prog_config object while we modify it
-	Glib::Mutex::Lock lock(*prog_config.mutex_p);
+	Thread::Mutex::Lock lock(*prog_config.mutex_p);
 	*prog_config.receive_dirname = 0;
       }
       return;
@@ -701,7 +693,7 @@ void EfaxController::receive(State mode) {
 
     fax_pathname += prog_config.receive_dirname;
 
-    stdout_pipe.open(Pipe_fifo::non_block);
+    stdout_pipe.open(PipeFifo::non_block);
     // efax is sensitive to timing, so set pipe write to non-block also
     stdout_pipe.make_write_non_block();
     
@@ -715,16 +707,15 @@ void EfaxController::receive(State mode) {
 
     // set up a synchronising pipe  in case the child process finishes before
     // fork() in parent space has returned (yes, with an exec() error that can
-    // happen with Linux kernel 2.6) - this is important because we test child_pid
-    // in efax_controller_childexit_handler()
-    Sync_pipe sync_pipe;
+    // happen with Linux kernel 2.6)
+    SyncPipe sync_pipe;
 
-    child_pid = fork();
-    if (child_pid == -1) {
+    prog_config.efax_controller_child_pid = fork();
+    if (prog_config.efax_controller_child_pid == -1) {
       write_error("Fork error - exiting\n");
       std::exit(FORK_ERROR);
     }
-    if (!child_pid) {  // child process - as soon as everything is set up we are going to do an exec()
+    if (!prog_config.efax_controller_child_pid) {  // child process - as soon as everything is set up we are going to do an exec()
 
       // now we have forked, we can connect stdout_pipe to stdout
       // and connect MainWindow::error_pipe to stderr
@@ -772,23 +763,7 @@ void EfaxController::delete_parms(std::pair<const char*, char* const*> parms_pai
   delete[] parms_pair.second;
 }
 
-bool EfaxController::read_pipe_slot(Glib::IOCondition) {
-
-  char pipe_buffer[PIPE_BUF + 1];
-  ssize_t result;
-
-  while ((result = stdout_pipe.read(pipe_buffer, PIPE_BUF)) > 0) {
-    pipe_buffer[result] = 0;
-    stdout_message(pipe_buffer);
-  }
-  if (result == -1                             // as this is a non-blocking pipe, result should
-      && (errno == EAGAIN || errno == EINTR)) {// end up being -1 with errno == EAGAIN
-    return true;                               // retain SigC connection
-  }
-  return false; // something has gone wrong (probably the pipe has been destroyed), so disconnect
-}
-
-void EfaxController::stop_slot(void) {
+void EfaxController::stop(void) {
  if (state != inactive) {
    stdout_message(gettext("\n*** Stopping send/receive session ***\n\n"));
    kill_child();
@@ -804,9 +779,9 @@ void EfaxController::timer_event(void) {
   // reap the status of any exited child processes
   while ((result = waitpid(-1, &stat_val, WNOHANG)) > 0) {
 
-    if (result && result == child_pid) {
+    if (result && result == prog_config.efax_controller_child_pid) {
 
-      child_pid = 0;
+      prog_config.efax_controller_child_pid = 0;
 
       int exit_code = -1;
       if (WIFEXITED(stat_val)) exit_code = WEXITSTATUS(stat_val);
@@ -825,7 +800,7 @@ void EfaxController::timer_event(void) {
 	  save_sent_fax();
 	  if (last_fax_item_sent.is_socket_file) {
 	    // if we are sending a print job via the socket server, notify the
-	    // Socket_server object which created the temporary file (the server
+	    // SocketServer object which created the temporary file (the server
 	    // object will clean up by deleting it and also cause the socket
 	    // file list to be updated - we do not need to do that here)
 	    remove_from_socket_server_filelist(last_fax_item_sent.file_list[0]);
@@ -918,39 +893,64 @@ void EfaxController::clear_receive_dirname(void) {
       else fax_received_notify();
     }
     // lock the Prog_config object while we modify it
-    Glib::Mutex::Lock lock(*prog_config.mutex_p);
+    Thread::Mutex::Lock lock(*prog_config.mutex_p);
     *prog_config.receive_dirname = 0;
   }
 }
 
+bool EfaxController::read_pipe_slot(void) {
+
+  char pipe_buffer[PIPE_BUF + 1];
+  ssize_t result;
+
+  while ((result = stdout_pipe.read(pipe_buffer, PIPE_BUF)) > 0) {
+    pipe_buffer[result] = 0;
+    stdout_message(pipe_buffer);
+  }
+  return true; // this is multi-shot
+}
+
 void EfaxController::join_child(void) {
   // we do not need a mutex for this - join_child() and unjoin_child() are
   // only called in the initial (GUI) thread
-  stdout_connection = Glib::signal_io().connect(sigc::mem_fun(*this, &EfaxController::read_pipe_slot),
-						stdout_pipe.get_read_fd(), Glib::IO_IN);
+
+  iowatch_tag = start_iowatch(stdout_pipe.get_read_fd(),
+			      sigc::mem_fun(*this, &EfaxController::read_pipe_slot),
+			      G_IO_IN);
 }
 
 void EfaxController::unjoin_child(void) {
   // we do not need a mutex for this - join_child() and unjoin_child() are
   // only called in the initial (GUI) thread, and stdout_pipe is only manipulated
   // in that thread or after a new single-threaded process has been launched with fork()
+
+  g_source_remove(iowatch_tag);
   stdout_pipe.close();
-  stdout_connection.disconnect();
 }
 
 void EfaxController::kill_child(void) {
-  if (child_pid) {
-    kill(child_pid, SIGTERM);
+  // it shouldn't be necessary to check whether the process group id of
+  // the process whose pid is in prog_config.efax_controller_child_pid
+  // is the same as efax-gtk (that is, prog_config.efax_controller_child_pid
+  // is not out of date) because it shouldn't happen - even if efax has ended
+  // we cannot have reaped its exit value yet if prog_config.efax_controller_child_pid
+  // is not 0 so it should still be a zombie in the process table, but for
+  // safety's sake let's just check it
+  if (prog_config.efax_controller_child_pid > 0
+      && getpgid(0) == getpgid(prog_config.efax_controller_child_pid)) {
+    kill(prog_config.efax_controller_child_pid, SIGTERM);
     // one second delay
-    Glib::usleep(1000000);
-    // now really make sure
-    // (we don't need to check child_pid again in the implementation as at
-    // version 2.2.12 because the child exit handling is in this thread, via
-    // MainWindow::timer_event_handler() and EfaxController::timer_event()
-    // but we would need to check it again (and protect it with a mutex) if
-    // the program is modified to put child exit handling in a signal handling
-    // thread waiting on sigwait() which resets the value of child_pid)
-    kill(child_pid, SIGKILL);
+    g_usleep(1000000);
+    // now really make sure (we don't need to check
+    // prog_config.efax_controller_child_pid again in the
+    // implementation as at version 2.2.12 because the child exit
+    // handling is in this thread, via MainWindow::timer_event_handler()
+    // and EfaxController::timer_event() but we would need to check it
+    // again (and protect it with a mutex) if the program is modified
+    // to put child exit handling in a signal handling thread waiting
+    // on sigwait() which resets the value of
+    // prog_config.efax_controller_child_pid)
+    kill(prog_config.efax_controller_child_pid, SIGKILL);
   }
 }
 
@@ -1212,3 +1212,52 @@ bool EfaxController::is_receiving_fax(void) const {
   }
   return return_val;
 }
+
+void EfaxController::cleanup_on_start(void) {
+
+  // this program cleans up for closure by ordinary user action, and
+  // atexit_cleanup() in main.cpp caters for cases where the X session
+  // has brought about the closure of this program.  It is also possible
+  // that no clean-up at all will occur if the program is terminated on,
+  // say, a power failure.  In that case we may be left with an empty
+  // fax directory in the faxin directory - if so, let's delete it
+
+#ifndef PATH_MAX
+#define PATH_MAX 1023
+#endif
+  int count;
+  void* success;
+  std::string old_working_dir;
+  for (count = 1, success = 0; !success && count < 10; count++) {
+    ScopedHandle<char*> path_buffer_h(new char[(PATH_MAX * count) + 1]);
+    success = getcwd(path_buffer_h.get(), PATH_MAX * count);
+    if (success) old_working_dir = path_buffer_h.get();
+  }
+
+  if (success) {
+    std::string new_dir(prog_config.working_dir);
+    new_dir += "/faxin";
+    chdir(new_dir.c_str());
+
+    DIR* dir_p;
+    if ((dir_p = opendir(new_dir.c_str()))) {
+
+      struct dirent* direntry;
+      struct stat statinfo;
+
+      while ((direntry = readdir(dir_p)) != 0) {
+	stat(direntry->d_name, &statinfo);
+	if (S_ISDIR(statinfo.st_mode)
+	    // keep the check for the "oldfax" sub-directory even though it is not used
+	    // in version 2.2.13 onwards (users may still have it lying around)
+	    && std::strcmp(direntry->d_name, "oldfax")
+	    && std::strcmp(direntry->d_name, ".")
+	    && std::strcmp(direntry->d_name, "..")) {
+	  // rmdir only deletes a directory if it is empty
+	  rmdir(direntry->d_name);
+	}
+      }
+    }
+    chdir(old_working_dir.c_str());
+  }
+}
