@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 2002 2003 Chris Vine
+/* Copyright (C) 2001 to 2005 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -6,9 +6,19 @@ COPYING distributed with the source files.
 
 */
 
+#include <gtk/gtkbutton.h>
+#include <gtk/gtkbox.h>
+#include <gtk/gtkvbox.h>
+#include <gtk/gtkbbox.h>
+#include <gtk/gtkhbbox.h>
+#include <gtk/gtkstock.h>
+#include <gtk/gtktextview.h>
+#include <gtk/gtktextbuffer.h>
+#include <gtk/gtkscrolledwindow.h>
+#include <gtk/gtklabel.h>
+
+#include <gdk/gdkpixbuf.h>
 #include <gdk/gdkkeysyms.h> // the key codes are here
-#include <gdkmm/pixbuf.h>
-#include <gtkmm/stock.h>
 
 #include "helpfile.h"
 
@@ -18,98 +28,105 @@ COPYING distributed with the source files.
 
 int HelpDialog::is_helpfile = 0;
 
+namespace { // callbacks for internal use only
 
-HelpDialog::HelpDialog(const int standard_size): Gtk::Window(Gtk::WINDOW_TOPLEVEL),
-			     close_button(Gtk::Stock::CLOSE),
-			     button_box(Gtk::BUTTONBOX_SPREAD) { 
+void HelpDialogCB::help_button_clicked(GtkWidget*, void* data) {
+  static_cast<HelpDialog*>(data)->close();
+}
+
+gboolean HelpDialogCB::help_key_press_event(GtkWidget*, GdkEventKey* event_p, void* data) {
+  HelpDialog* instance_p = static_cast<HelpDialog*>(data);
+  int keycode = event_p->keyval;
+  
+  if (keycode == GDK_Escape) instance_p->close();
+  else if (keycode == GDK_Home || keycode == GDK_End
+	   || keycode == GDK_Up || keycode == GDK_Down
+	   || keycode == GDK_Page_Up || keycode == GDK_Page_Down) {
+    gtk_widget_event(gtk_notebook_get_nth_page(instance_p->notebook_p,
+					       gtk_notebook_get_current_page(instance_p->notebook_p)),
+					       (GdkEvent*)event_p);
+  }
+  return true;    // stop processing here
+}
+
+}  // anonymous namespace
+
+HelpDialog::HelpDialog(const int standard_size): 
+                             WinBase(gettext("efax-gtk: Help"),
+				     prog_config.window_icon_h) {
 
   // notify the existence of this object
   is_helpfile++;
 
-  sending_editbox.set_wrap_mode(Gtk::WRAP_WORD);
-  sending_editbox.set_editable(false);
-  sending_editbox.modify_font(Pango::FontDescription(prog_config.fixed_font));
-  sending_editbox.get_buffer()->set_text(get_sending_help());
-  sending_editbox.unset_flags(Gtk::CAN_FOCUS);
-
-  sending_scrolled_window.set_shadow_type(Gtk::SHADOW_IN);
-  sending_scrolled_window.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
-  sending_scrolled_window.add(sending_editbox);
-
-  receiving_editbox.set_wrap_mode(Gtk::WRAP_WORD);
-  receiving_editbox.set_editable(false);
-  receiving_editbox.modify_font(Pango::FontDescription(prog_config.fixed_font));
-  receiving_editbox.get_buffer()->set_text(get_receiving_help());
-  receiving_editbox.unset_flags(Gtk::CAN_FOCUS);
-
-  receiving_scrolled_window.set_shadow_type(Gtk::SHADOW_IN);
-  receiving_scrolled_window.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
-  receiving_scrolled_window.add(receiving_editbox);
-
-  addressbook_editbox.set_wrap_mode(Gtk::WRAP_WORD);
-  addressbook_editbox.set_editable(false);
-  addressbook_editbox.modify_font(Pango::FontDescription(prog_config.fixed_font));
-  addressbook_editbox.get_buffer()->set_text(get_addressbook_help());
-  addressbook_editbox.unset_flags(Gtk::CAN_FOCUS);
-
-  addressbook_scrolled_window.set_shadow_type(Gtk::SHADOW_IN);
-  addressbook_scrolled_window.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
-  addressbook_scrolled_window.add(addressbook_editbox);
-
-  fax_list_editbox.set_wrap_mode(Gtk::WRAP_WORD);
-  fax_list_editbox.set_editable(false);
-  fax_list_editbox.modify_font(Pango::FontDescription(prog_config.fixed_font));
-  fax_list_editbox.get_buffer()->set_text(get_fax_list_help());
-  fax_list_editbox.unset_flags(Gtk::CAN_FOCUS);
-
-  fax_list_scrolled_window.set_shadow_type(Gtk::SHADOW_IN);
-  fax_list_scrolled_window.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
-  fax_list_scrolled_window.add(fax_list_editbox);
-
-  settings_editbox.set_wrap_mode(Gtk::WRAP_WORD);
-  settings_editbox.set_editable(false);
-  settings_editbox.modify_font(Pango::FontDescription(prog_config.fixed_font));
-  settings_editbox.get_buffer()->set_text(get_settings_help());
-  settings_editbox.unset_flags(Gtk::CAN_FOCUS);
-
-  settings_scrolled_window.set_shadow_type(Gtk::SHADOW_IN);
-  settings_scrolled_window.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
-  settings_scrolled_window.add(settings_editbox);
-
-  button_box.add(close_button);
-
-  notebook.set_tab_pos(Gtk::POS_TOP);
-  notebook.set_scrollable(true);
-  // set up the notebook pages
-  {
-    using namespace Gtk::Notebook_Helpers;
-    PageList& page_list = notebook.pages();
-    page_list.push_back(TabElem(sending_scrolled_window, gettext("Sending")));
-    page_list.push_back(TabElem(receiving_scrolled_window, gettext("Receiving")));
-    page_list.push_back(TabElem(addressbook_scrolled_window, gettext("Address Book")));
-    page_list.push_back(TabElem(fax_list_scrolled_window, gettext("Fax Lists")));
-    page_list.push_back(TabElem(settings_scrolled_window, gettext("Settings")));
-  }
-
-  vbox.pack_start(notebook, Gtk::PACK_EXPAND_WIDGET, standard_size/3);
-  vbox.pack_start(button_box, Gtk::PACK_SHRINK, standard_size/3);
-
-  close_button.signal_clicked().connect(sigc::mem_fun(*this, &HelpDialog::kill_dialog));
+  GtkWidget* close_button_p = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
   
-  add(vbox);
-  
-  set_title(gettext("efax-gtk: Help"));
-  set_default_size(standard_size * 25, standard_size * 14);
-  
-  set_border_width(standard_size/4);
-  //set_position(Gtk::WIN_POS_NONE);
-  
-  grab_focus();
-  close_button.unset_flags(Gtk::CAN_FOCUS);
+  GtkWidget* button_box_p = gtk_hbutton_box_new();
+  gtk_button_box_set_layout(GTK_BUTTON_BOX(button_box_p), GTK_BUTTONBOX_SPREAD);
+  gtk_container_add(GTK_CONTAINER(button_box_p), close_button_p);
 
-  set_icon(prog_config.window_icon_r);
+  notebook_p = GTK_NOTEBOOK(gtk_notebook_new());
+  GtkWidget* scrolled_window_p;
+  GtkWidget* label_p;
 
-  show_all();
+  // make and append sending help page
+  scrolled_window_p = make_scrolled_window();
+  gtk_container_add(GTK_CONTAINER(scrolled_window_p),
+		    make_text_view(get_sending_help()));
+  label_p = gtk_label_new(gettext("Sending"));
+  gtk_notebook_append_page(notebook_p, scrolled_window_p, label_p);
+
+  // make and append receiving help page
+  scrolled_window_p = make_scrolled_window();
+  gtk_container_add(GTK_CONTAINER(scrolled_window_p),
+		    make_text_view(get_receiving_help()));
+  label_p = gtk_label_new(gettext("Receiving"));
+  gtk_notebook_append_page(notebook_p, scrolled_window_p, label_p);
+
+  // make and append addressbook help page
+  scrolled_window_p = make_scrolled_window();
+  gtk_container_add(GTK_CONTAINER(scrolled_window_p),
+		    make_text_view(get_addressbook_help()));
+  label_p = gtk_label_new(gettext("Address Book"));
+  gtk_notebook_append_page(notebook_p, scrolled_window_p, label_p);
+
+  // make and append fax list help page
+  scrolled_window_p = make_scrolled_window();
+  gtk_container_add(GTK_CONTAINER(scrolled_window_p),
+		    make_text_view(get_fax_list_help()));
+  label_p = gtk_label_new(gettext("Fax Lists"));
+  gtk_notebook_append_page(notebook_p, scrolled_window_p, label_p);
+
+  // make and append settings help page
+  scrolled_window_p = make_scrolled_window();
+  gtk_container_add(GTK_CONTAINER(scrolled_window_p),
+		    make_text_view(get_settings_help()));
+  label_p = gtk_label_new(gettext("Settings"));
+  gtk_notebook_append_page(notebook_p, scrolled_window_p, label_p);
+
+  gtk_notebook_set_tab_pos(notebook_p, GTK_POS_TOP);
+  gtk_notebook_set_scrollable(notebook_p, true);
+
+  GtkBox* vbox_p = GTK_BOX(gtk_vbox_new(false, 0));
+  gtk_box_pack_start(vbox_p, GTK_WIDGET(notebook_p), true, true, standard_size/3);
+  gtk_box_pack_start(vbox_p, button_box_p, false, false, standard_size/3);
+
+  g_signal_connect(G_OBJECT(close_button_p), "clicked",
+		   G_CALLBACK(HelpDialogCB::help_button_clicked), this);
+  g_signal_connect(G_OBJECT(get_win()), "key_press_event",
+		   G_CALLBACK(HelpDialogCB::help_key_press_event), this);
+
+  gtk_container_add(GTK_CONTAINER(get_win()), GTK_WIDGET(vbox_p));
+
+  gtk_window_set_default_size(get_win(), standard_size * 25, standard_size * 14);
+  
+  gtk_container_set_border_width(GTK_CONTAINER(get_win()), standard_size/4);
+  //gtk_window_set_position(get_win(), GTK_WIN_POS_NONE);
+  
+  gtk_widget_grab_focus(GTK_WIDGET(get_win()));
+
+  GTK_WIDGET_UNSET_FLAGS(close_button_p, GTK_CAN_FOCUS);
+
+  gtk_widget_show_all(GTK_WIDGET(get_win()));
 }
 
 HelpDialog::~HelpDialog(void) {
@@ -117,29 +134,29 @@ HelpDialog::~HelpDialog(void) {
   is_helpfile--;
 }
 
-void HelpDialog::kill_dialog(void) {
-  hide_all();
-  delete this;  // this a modeless dialog, and so safe
+GtkWidget* HelpDialog::make_text_view(const char* text) {
+
+  GtkTextView* text_view_p = GTK_TEXT_VIEW(gtk_text_view_new());
+  gtk_text_view_set_wrap_mode(text_view_p, GTK_WRAP_WORD);
+  gtk_text_view_set_editable(text_view_p, false);
+  PangoFontDescription* font_description = 
+    pango_font_description_from_string(prog_config.fixed_font.c_str());
+  gtk_widget_modify_font(GTK_WIDGET(text_view_p), font_description);
+  pango_font_description_free(font_description);
+  gtk_text_buffer_set_text(gtk_text_view_get_buffer(text_view_p),
+			   text,
+			   -1);
+  GTK_WIDGET_UNSET_FLAGS(GTK_WIDGET(text_view_p), GTK_CAN_FOCUS);
+  return GTK_WIDGET(text_view_p);
 }
 
-bool HelpDialog::on_delete_event(GdkEventAny*) {
-  kill_dialog();
-  return true; // returning true prevents destroy sig being emitted
-}
+GtkWidget* HelpDialog::make_scrolled_window(void) {
 
-bool HelpDialog::on_key_press_event(GdkEventKey* event_p) {
-
-  int keycode = event_p->keyval;
-  
-  if (keycode == GDK_Escape) kill_dialog();
-
-  else if (keycode == GDK_Home || keycode == GDK_End
-	   || keycode == GDK_Up || keycode == GDK_Down
-	   || keycode == GDK_Page_Up || keycode == GDK_Page_Down) {
-    static_cast<HelpEditor*>(notebook.get_nth_page(notebook.get_current_page()))->on_key_press_event(event_p);
-    return false;
-  }
-  return true;
+  GtkScrolledWindow* scrolled_window_p = GTK_SCROLLED_WINDOW(gtk_scrolled_window_new(0, 0));
+  gtk_scrolled_window_set_shadow_type(scrolled_window_p, GTK_SHADOW_IN);
+  gtk_scrolled_window_set_policy(scrolled_window_p, GTK_POLICY_AUTOMATIC,
+				 GTK_POLICY_AUTOMATIC);
+  return GTK_WIDGET(scrolled_window_p);
 }
 
 const char* HelpDialog::get_sending_help(void) {
