@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 to 2004 Chris Vine
+/* Copyright (C) 2001 to 2005 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -22,26 +22,50 @@ COPYING distributed with the source files.
 #include <string>
 #include <cstring>
 
-#include <gtkmm/main.h>
-#include <gtkmm/window.h>
-#include <glibmm/ustring.h>
-#include <glibmm/convert.h>
-#include <glibmm/thread.h>
+#include <gtk/gtkmain.h>
+#include <gtk/gtkversion.h>
+
+#include <gdk/gdkpixbuf.h>
 #include <gdk/gdkx.h>  // for GDK_DISPLAY()
 #include <gdk/gdk.h>   // for gdk_notify_startup_complete()
 #include <X11/Xlib.h>
 
+#include <glib/gutils.h>
+#include <glib/gthread.h>
+
+// a guess for the future!
+#if GTK_MAJOR_VERSION > 2
+#define EFAX_GTK_USE_ICON_THEME 1
+// a pick for the present
+#elif GTK_MINOR_VERSION >= 4
+#define EFAX_GTK_USE_ICON_THEME 1
+#else
+#undef EFAX_GTK_USE_ICON_THEME
+#endif
+
+// a guess for the future!
+#if GTK_MAJOR_VERSION > 2
+#define HAVE_NOTIFY_STARTUP 1
+// a pick for the present
+#elif GTK_MINOR_VERSION >= 2
+#define HAVE_NOTIFY_STARTUP 1
+#else
+#undef HAVE_NOTIFY_STARTUP
+#endif
+
+#ifdef EFAX_GTK_USE_ICON_THEME
+#include <gtk/gtkicontheme.h>
+#include "icon_info_handle.h"
+#endif
+
 #include "mainwindow.h"
 #include "dialogs.h"
+#include "mutex.h"
 #include "window_icon.h"
+#include "utf8_utils.h"
+#include "gobj_handle.h"
 #include "fdstream/fdstream.h"
 
-#if GTKMM_VERSION >= 24
-#include <gtk/gtkicontheme.h>
-#include <gtkmm/icontheme.h>
-#include "gtk_icon_info_handle.h"
-#endif
-
 #ifdef ENABLE_NLS
 #include <locale.h>
 #include <libintl.h>
@@ -49,20 +73,25 @@ COPYING distributed with the source files.
 
 #define LOCKFILE_NAME "/.efax-gtk-pid"
 
+// the headers on one of my machines does not properly include the declaration
+// of getpgid() for some reason, so declare it here explicitly
+extern "C" pid_t getpgid(pid_t);
+
 Prog_config prog_config;
 
 bool is_program_running(void);
 
-bool get_prog_parm(const char*, std::string&, Glib::ustring&, Glib::ustring(*)(const std::string&));
+bool get_prog_parm(const char*, std::string&, std::string&, std::string(*)(const std::string&));
 
-inline bool get_prog_parm(const char* name, std::string& line, Glib::ustring& result) {
-  return get_prog_parm(name, line, result, Glib::locale_to_utf8);
+inline bool get_prog_parm(const char* name, std::string& line, std::string& result) {
+  return get_prog_parm(name, line, result, Utf8::locale_to_utf8);
 }
 
 void get_fonts(void);
 void get_window_icon(void);
 int mkdir_with_parent(const std::string&);
 bool is_arg(const char*, int, char*[]);
+extern "C" void atexit_cleanup(void);
 
 int main(int argc, char* argv[]) {
 
@@ -74,6 +103,14 @@ int main(int argc, char* argv[]) {
   setlocale(LC_ALL,"");
 #endif
 
+  // provide a new process group with this process as the process group leader
+  // so we can check whether another process is one of our children (ie a member
+  // of our process group) and also call kill(0, SIGTERM) in atexit_cleanup()
+  if (setpgid(getpid(), getpid()) == -1) {
+    std::string message("Error in creating new process group\n");
+    write(2, message.data(), message.size());
+  }
+
   if (argc > 1) {
     if (is_arg("--version", argc, argv)) {
       std::string message("efax-gtk-" VERSION "\n");
@@ -81,10 +118,10 @@ int main(int argc, char* argv[]) {
 			 "This program is released under the "
 			 "GNU General Public License, version 2\n");
       try {
-	message = Glib::locale_from_utf8(message);
+	message = Utf8::locale_from_utf8(message);
 	write(1, message.data(), message.size());
       }
-      catch (Glib::ConvertError&) {
+      catch (Utf8::ConversionError&) {
 	std::string err_message("UTF-8 conversion error in main()\n");
 	write(2, err_message.data(), err_message.size());
       }
@@ -100,10 +137,10 @@ int main(int argc, char* argv[]) {
 			 "See the README file which comes with the distribution\n"
 			 "for further details\n");
       try {
-	message = Glib::locale_from_utf8(message);
+	message = Utf8::locale_from_utf8(message);
 	write(1, message.data(), message.size());
       }
-      catch  (Glib::ConvertError&) {
+      catch  (Utf8::ConversionError&) {
 	std::string err_message("UTF-8 conversion error in main()\n");
 	write(2, err_message.data(), err_message.size());
       }
@@ -111,14 +148,17 @@ int main(int argc, char* argv[]) {
     }
   }
 
-  Glib::thread_init();
-  prog_config.mutex_p = new Glib::Mutex;
-
+  // although we use Thread::Thread and Thread::Mutex classes based on pthreads,
+  // we need tocall g_thread_init() in order to make glib thread safe
+  g_thread_init(0);
+  prog_config.mutex_p = new Thread::Mutex;
+  
   std::string messages(configure_prog(false));
 
   if(!is_program_running()) {
 
-    Gtk::Main app(argc, argv);
+    gtk_init(&argc, &argv);
+    g_atexit(atexit_cleanup);
 
     get_fonts();
 
@@ -166,45 +206,30 @@ int main(int argc, char* argv[]) {
 
       // everything is set up
       // now enter the main program loop
-      app.run();
+      gtk_main();
     }
-    // remove program lock file
-    std::string lockfile(prog_config.working_dir);
-    lockfile += LOCKFILE_NAME;
-    unlink(lockfile.c_str());
   }
+#ifdef HAVE_NOTIFY_STARTUP
   else {
     // if we have got here then is_program_running() will have detected
     // that there is already an instance of this program running and
-    // it will have sent a signal to that instance to present itself
-    // on the current workspace.  We now need to tell the window
+    // it will have sent a message to that instance via a fifo to present
+    // itself on the current workspace.  We now need to tell the window
     // manager that we have completed startup, or the window manager
     // startup notifier will be confused
-    Gtk::Main app(argc, argv);
-#if GTKMM_VERSION >= 22
-    // if we have version 2.2 or higher of gtkmm then we must have
-    // version 2.2 or higher of gtk+, so gdk_notify_startup_complete()
-    // is available
+    // gdk_notify_startup_complete() is first available with GTK+-2.2
+    gtk_init(&argc, &argv);
     gdk_notify_startup_complete();
-#else
-    // if we are using version 2.0 of gtkmm then we must have version
-    // 2.0 of gtk+ so we will do a hack to notify the window manager
-    // by starting a dummy window (however we never get into a program
-    // loop so it finishes immediately after it has started)
-    Gtk::Window window(Gtk::WINDOW_POPUP);
-    window.set_size_request(1,1);
-    window.show();
-    window.hide();
-#endif
   }
+#endif
   delete prog_config.mutex_p;
   return 0;
 }
 
-Glib::ustring configure_prog(bool reread) {
+std::string configure_prog(bool reread) {
 
   // lock the Prog_config object while we modify it
-  Glib::Mutex::Lock lock(*prog_config.mutex_p);
+  Thread::Mutex::Lock lock(*prog_config.mutex_p);
 
   if (!reread) {
 
@@ -222,7 +247,7 @@ Glib::ustring configure_prog(bool reread) {
   prog_config.found_rcfile = false;
   std::ifstream filein;
   std::string rcfile;
-  Glib::ustring return_val;
+  std::string return_val;
 
   if (!prog_config.homedir.empty()) {
     rcfile = prog_config.homedir;
@@ -279,12 +304,12 @@ Glib::ustring configure_prog(bool reread) {
     }
     if (!prog_config.homedir.empty()) {
       try {
-	Glib::ustring temp(Glib::filename_to_utf8(prog_config.homedir));
+	std::string temp(Utf8::filename_to_utf8(prog_config.homedir));
 	return_val +=  "or ";
 	return_val += temp;
 	return_val += "/." RC_FILE;
       }
-      catch (Glib::ConvertError&) {
+      catch (Utf8::ConversionError&) {
 	write_error("UTF-8 conversion error in configure_prog()\n");
       }
     }
@@ -310,28 +335,32 @@ Glib::ustring configure_prog(bool reread) {
       prog_config.logfile_name = "";
     }
     // if we are setting up for first time, initialise prog_config.receive_dirname
-    else *prog_config.receive_dirname = 0;
+    // and prog_config.efax_controller_child_pid;
+    else {
+      *prog_config.receive_dirname = 0;
+      prog_config.efax_controller_child_pid = 0;
+    }
 
 // now extract settings from file
 
     std::string file_read;
-    Glib::ustring device;
-    Glib::ustring lock_file;
-    Glib::ustring modem_class;
-    Glib::ustring rings;
-    Glib::ustring dialmode;
-    Glib::ustring init;
-    Glib::ustring reset;
-    Glib::ustring capabilities;
-    Glib::ustring extra_parms;
-    Glib::ustring print_popup;
-    Glib::ustring sock_server;
-    Glib::ustring sock_popup;
-    Glib::ustring sock_client_address;
-    Glib::ustring sock_other_addresses;
-    Glib::ustring fax_received_popup;
-    Glib::ustring fax_received_exec;
-    Glib::ustring work_subdir;
+    std::string device;
+    std::string lock_file;
+    std::string modem_class;
+    std::string rings;
+    std::string dialmode;
+    std::string init;
+    std::string reset;
+    std::string capabilities;
+    std::string extra_parms;
+    std::string print_popup;
+    std::string sock_server;
+    std::string sock_popup;
+    std::string sock_client_address;
+    std::string sock_other_addresses;
+    std::string fax_received_popup;
+    std::string fax_received_exec;
+    std::string work_subdir;
     
     while (std::getline(filein, file_read)) {
 
@@ -351,7 +380,7 @@ Glib::ustring configure_prog(bool reread) {
 	
 	// look for "LOCK:"
 	else if (get_prog_parm("LOCK:", file_read, lock_file,
-			       Glib::filename_to_utf8));
+			       Utf8::filename_to_utf8));
 
 	// look for "CLASS:"
 	else if (get_prog_parm("CLASS:", file_read, modem_class));
@@ -415,15 +444,15 @@ Glib::ustring configure_prog(bool reread) {
 
 	// look for "FAX_RECEIVED_PROG:"
 	else if (get_prog_parm("FAX_RECEIVED_PROG:", file_read, prog_config.fax_received_prog,
-			       Glib::filename_to_utf8));
+			       Utf8::filename_to_utf8));
 
 	// look for "LOG_FILE:"
 	else if (get_prog_parm("LOG_FILE:", file_read, prog_config.logfile_name,
-			       Glib::filename_to_utf8));
+			       Utf8::filename_to_utf8));
 
 	// look for "WORK_SUBDIR:"
 	else if (get_prog_parm("WORK_SUBDIR:", file_read, work_subdir,
-			       Glib::filename_to_utf8));
+			       Utf8::filename_to_utf8));
       }
     }
 
@@ -522,9 +551,9 @@ Glib::ustring configure_prog(bool reread) {
     if (!device.empty()) {
       std::string locale_device;
       try {
-	locale_device = Glib::locale_from_utf8(device);
+	locale_device = Utf8::locale_from_utf8(device);
       }
-      catch (Glib::ConvertError&) {
+      catch (Utf8::ConversionError&) {
 	write_error("UTF-8 conversion error in configure_prog() - device\n");
       }
 
@@ -534,9 +563,9 @@ Glib::ustring configure_prog(bool reread) {
 	}
 	else {
 	  try {
-	    prog_config.lock_file = Glib::filename_from_utf8(lock_file);
+	    prog_config.lock_file = Utf8::filename_from_utf8(lock_file);
 	  }
-	  catch (Glib::ConvertError&) {
+	  catch (Utf8::ConversionError&) {
 	    write_error("UTF-8 conversion error in configure_prog() - lock file\n");
 	    write_error("Defaulting to /var/lock\n");
 	    prog_config.lock_file = "/var/lock";
@@ -567,7 +596,7 @@ Glib::ustring configure_prog(bool reread) {
       prog_config.resolution = "204x196";
     }
     else {
-      temp = prog_config.resolution.lowercase();
+      temp = Utf8::to_lower(prog_config.resolution);
       if (!temp.compare("fine")) prog_config.resolution = "204x196";
       else if (!temp.compare("standard")) prog_config.resolution = "204x98";
       else {
@@ -591,7 +620,7 @@ Glib::ustring configure_prog(bool reread) {
       prog_config.page_dim = "210x297mm";
     }
     else {
-      prog_config.page_size = prog_config.page_size.lowercase();
+      prog_config.page_size = Utf8::to_lower(prog_config.page_size);
       if (!prog_config.page_size.compare("a4")) prog_config.page_dim = "210x297mm";
       else if (!prog_config.page_size.compare("letter")) prog_config.page_dim = "216x279mm";
       else if (!prog_config.page_size.compare("legal")) prog_config.page_dim = "216x356mm";
@@ -605,7 +634,7 @@ Glib::ustring configure_prog(bool reread) {
     
     if (dialmode.empty()) prog_config.tone_dial = true;
     else {
-      temp = dialmode.lowercase();
+      temp = Utf8::to_lower(dialmode);
       if (!temp.compare("tone")) prog_config.tone_dial = true;
       else if (!temp.compare("pulse")) prog_config.tone_dial = false;
       else {
@@ -645,7 +674,7 @@ Glib::ustring configure_prog(bool reread) {
       prog_config.print_shrink = "100";
     }
     
-    temp = print_popup.lowercase();
+    temp = Utf8::to_lower(print_popup);
     if (!temp.compare("no")) prog_config.print_popup = false;
     else prog_config.print_popup = true;
 
@@ -654,11 +683,11 @@ Glib::ustring configure_prog(bool reread) {
       prog_config.ps_view_cmd = "gv";
     }
 
-    temp = sock_server.lowercase();
+    temp = Utf8::to_lower(sock_server);
     if (!temp.compare("yes")) prog_config.sock_server = true;
     else prog_config.sock_server = false;
 
-    temp = sock_popup.lowercase();
+    temp = Utf8::to_lower(sock_popup);
     if (!temp.compare("yes")) prog_config.sock_popup = true;
     else prog_config.sock_popup = false;
 
@@ -676,7 +705,7 @@ Glib::ustring configure_prog(bool reread) {
       prog_config.sock_server_port = "";
     }
 
-    temp = sock_client_address.lowercase();
+    temp = Utf8::to_lower(sock_client_address);
     if (!temp.compare("other")) prog_config.other_sock_client_address = true;
     else prog_config.other_sock_client_address = false;
 
@@ -698,11 +727,11 @@ Glib::ustring configure_prog(bool reread) {
       }
     }
 
-    temp = fax_received_popup.lowercase();
+    temp = Utf8::to_lower(fax_received_popup);
     if (!temp.compare("yes")) prog_config.fax_received_popup = true;
     else prog_config.fax_received_popup = false;
 
-    temp = fax_received_exec.lowercase();
+    temp = Utf8::to_lower(fax_received_exec);
     if (!temp.compare("yes")) prog_config.fax_received_exec = true;
     else prog_config.fax_received_exec = false;
 
@@ -712,7 +741,7 @@ Glib::ustring configure_prog(bool reread) {
       // CMV 24-07-04
       if (!work_subdir.empty()) {
 	prog_config.working_dir += '/';
-	prog_config.working_dir += Glib::filename_from_utf8(work_subdir);
+	prog_config.working_dir += Utf8::filename_from_utf8(work_subdir);
 	if (mkdir_with_parent(prog_config.working_dir) && errno != EEXIST) {
 	  return_val += gettext("Invalid WORK_SUBDIR: directory specified. "
 				"WORK_SUBDIR: will be ignored\n");
@@ -758,18 +787,17 @@ void get_fonts(void) {
   XFreeFontNames(fonts);
 }
 
-bool get_prog_parm(const char* name, std::string& line, Glib::ustring& result,
-		   Glib::ustring(*convert_func)(const std::string&)) {
+bool get_prog_parm(const char* name, std::string& line, std::string& result,
+		   std::string(*convert_func)(const std::string&)) {
 // This function looks for a setting named `name' in the string `line'
 // and returns the values stated after it in string `result'.  It returns
 // `true' if the setting was found.  If there are trailing spaces or tabs,
 // string `line' will be modified.  string `result' is only modified if
 // the `name' setting is found.  Anything extracted from `line' will be
-// converted (when placed into `result') to UTF-8 as maintained by
-// Glib::ustring, using the function assigned to function pointer
-// convert_func (you would normally use Glib::locale_to_utf8() or
-// Glib::filename_to_utf8(), and there is a default inline function
-// using Glib::locale_to_utf8()
+// converted (when placed into `result') to UTF-8, using the function
+// assigned to function pointer convert_func (you would normally use
+// Utf8::locale_to_utf8() or Utf8::filename_to_utf8(), and there is a
+// default inline function using Utf8::locale_to_utf8()
 
   const std::string::size_type length = std::strlen(name);
   // we have to use std::string::substr() because libstdc++-2
@@ -784,7 +812,7 @@ bool get_prog_parm(const char* name, std::string& line, Glib::ustring& result,
 	try {
 	  result.assign(convert_func(line.substr(pos)));
 	}
-	catch (Glib::ConvertError&) {
+	catch (Utf8::ConversionError&) {
 	  result = "";
 	  write_error("UTF-8 conversion error in get_prog_parm()\n");
 	}
@@ -803,37 +831,29 @@ void get_window_icon(void) {
 
   bool have_icon = false;
 
-#if GTKMM_VERSION >= 24
-  Glib::RefPtr<Gtk::IconTheme> icon_theme_r = Gtk::IconTheme::get_default();
+#ifdef EFAX_GTK_USE_ICON_THEME
 
-  // use the C function to look-up the icon - the gtkmm wrapper does not
-  // check whether look-up succeeded.  In addition, Gtk::IconInfo::get_filename()
-  // incorrectly returns a Glib::ustring object instead of a std::string object,
-  // without converting its contents to a valid UTF-8 encoded string
+  GtkIconTheme* icon_theme_p = gtk_icon_theme_get_default();
 
-  Gtk_icon_info_scoped_handle icon_info_h(
-			      gtk_icon_theme_lookup_icon(icon_theme_r->gobj(),
-							 "stock_send-fax",
-							 24, GtkIconLookupFlags(0)));
+  IconInfoScopedHandle icon_info_h(gtk_icon_theme_lookup_icon(icon_theme_p,
+							      "stock_send-fax",
+							      24, GtkIconLookupFlags(0)));
   if (icon_info_h.get()) {
       
-    // we could use Glib::convert_const_gchar_ptr_to_stdstring, but this is not
-    // part of the publicly documented interface
-    const char* icon_path_p = (const char*)gtk_icon_info_get_filename(icon_info_h.get());
+    const gchar* icon_path_p = gtk_icon_info_get_filename(icon_info_h.get());
     if (icon_path_p) {
-      try {
-	prog_config.window_icon_r = Gdk::Pixbuf::create_from_file(std::string(icon_path_p));
-	have_icon = true;
-      }
-      catch (Gdk::PixbufError&) {
-	write_error("Pixbuf error in get_window_icon()\n");
-      }
+      GError* error_p = 0;
+      prog_config.window_icon_h =
+	GobjHandle<GdkPixbuf>(gdk_pixbuf_new_from_file(icon_path_p, &error_p));
+      if (prog_config.window_icon_h.get()) have_icon = true;
+      else write_error("Pixbuf error in get_window_icon()\n");
     }
   }
 #endif
 
   if (!have_icon) {
-    prog_config.window_icon_r = Gdk::Pixbuf::create_from_xpm_data(window_icon_xpm);
+    prog_config.window_icon_h =
+      GobjHandle<GdkPixbuf>(gdk_pixbuf_new_from_xpm_data(window_icon_xpm));
   }
 }
 
@@ -894,6 +914,8 @@ bool is_program_running(void) {
     }
 
     else {   // the lockfile already exists - see if it is stale
+             // it shouldn't ever be stale with the other clean up provision
+             // in this program, but nonetheless ...
       ifstream existing_lock(lockfile.c_str(), ios::in);
       if(!existing_lock) { // can't open lock file
 	std::cerr << "Not able to access existing lockfile " << lockfile << std::endl;
@@ -909,11 +931,27 @@ bool is_program_running(void) {
 			     S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
 	  count++;                     // signal we can only have one more try if this open fails
 	}
-	else {                         // active lock - send efax-gtk a signal
+	else {                         // active lock - alert the program pipe
 	  std::cout << "Active lock - bringing up existing instance of efax-gtk" << std::endl;
 	  count = 2;                   // break out of the loop
-	  kill(existing_lock_pid, SIGUSR2);
-	  return_val = true;
+
+	  std::string fifo_name(prog_config.working_dir);
+	  fifo_name += PIPE_NAME;
+	  
+	  if (!access(fifo_name.c_str(), F_OK)) { // if we don't enter this block we have a
+                                                  // problem - the fifo should exist but doesn't
+	    int write_fd = open(fifo_name.c_str(), O_WRONLY | O_NONBLOCK);
+	    if (write_fd >= 0) {                  // if we don't enter this block we also have a
+                                                  // problem - the fifo can't be opened for writing
+	      Attachable::Out pipe_stream;
+	      pipe_stream.attach(write_fd);
+	      pipe_stream << existing_lock_pid << std::endl;
+	      pipe_stream.close();
+	      return_val = true;
+	    }
+	    else std::cerr << "Error: cannot open fifo for writing" << std::endl;
+	  }
+	  else std::cerr << "Error: program fifo doesn't exist" << std::endl;
 	}
       }
     }
@@ -927,3 +965,50 @@ bool is_program_running(void) {
   }
   return return_val;
 }
+
+void atexit_cleanup(void) {
+
+  // this does any essential clean up, which is always required to make the program
+  // restart correctly.  It is only relevant to a case where the program has closed
+  // down because X server connection has closed - where the program is closed by direct
+  // user input to the program, then the program cleans up elsewhere
+
+  // check we are the process group leader (we should be as setpgid() was
+  // called in main()) and if so kill any stray children, such as an efax
+  // instance or a fax viewing instance
+  if (getpgid(0) == getpid()) {
+
+    // block the signal to us
+    struct sigaction sig_act;
+    sig_act.sa_handler = SIG_IGN;
+    sigemptyset(&sig_act.sa_mask);
+    sig_act.sa_flags = 0;
+    sigaction(SIGTERM, &sig_act, 0);
+
+    // kill
+    kill(0, SIGTERM);
+  }
+  else std::cerr << "Error: we are not a process group leader!" << std::endl;
+
+  // remove program lock file
+  std::string lockfile(prog_config.working_dir);
+  lockfile += LOCKFILE_NAME;
+  unlink(lockfile.c_str());
+
+  // remove program fifo
+  std::string fifo_name(prog_config.working_dir);
+  fifo_name += PIPE_NAME;
+  unlink(fifo_name.c_str());
+
+  // delete receive_dirname if it is empty
+  if (*prog_config.receive_dirname && !prog_config.working_dir.empty()) {
+    std::string full_dirname(prog_config.working_dir);
+    full_dirname += "/faxin/";
+    full_dirname += prog_config.receive_dirname;
+    rmdir(full_dirname.c_str()); // only deletes the directory if it is empty
+  }
+
+  // call up a shell to dispense with any stray viewing files
+  // (these may exist if the program is exited when a fax is still being viewed)
+  std::system("rm -f /tmp/efax-gtk-view.*");
+}
