@@ -14,7 +14,7 @@ COPYING distributed with the source files.
 #include <semaphore.h>
 #include <errno.h>
 
-class Sem_sync {
+class SemSync {
 
   bool valid;
   sem_t sem;
@@ -24,27 +24,27 @@ public:
   int getvalue(void);
   int post(void) {return sem_post(&sem);}
   void wait(void);
-  Sem_sync(unsigned int init_count = 0);
-  ~Sem_sync(void);
+  SemSync(unsigned int init_count = 0);
+  ~SemSync(void);
 };
 
-inline int Sem_sync::getvalue(void) {
+inline int SemSync::getvalue(void) {
   int count;
   sem_getvalue(&sem, &count);
   return count;
 }
 
-inline void Sem_sync::wait(void) {
+inline void SemSync::wait(void) {
 
   while (sem_wait(&sem) == -1 && errno == EINTR);
 }
 
-inline Sem_sync::Sem_sync(unsigned int init_count): valid(false) {
+inline SemSync::SemSync(unsigned int init_count): valid(false) {
 
   if(sem_init(&sem, 0, init_count) == 0) valid = true;
 }
 
-inline Sem_sync::~Sem_sync(void) {
+inline SemSync::~SemSync(void) {
 
   while (getvalue() < 1) post();
   sem_destroy(&sem);
