@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 2002 Chris Vine
+/* Copyright (C) 2001 2002 and 2005 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -6,9 +6,14 @@ COPYING distributed with the source files.
 
 */
 
-#include <gdkmm/pixbuf.h>
-#include <gtkmm/main.h>
-#include <gtkmm/stock.h>
+#include <gtk/gtkcontainer.h>
+#include <gtk/gtktable.h>
+#include <gtk/gtklabel.h>
+#include <gtk/gtkbutton.h>
+#include <gtk/gtkbbox.h>
+#include <gtk/gtkhbbox.h>
+#include <gtk/gtkstock.h>
+
 
 #include "settings_help.h"
 
@@ -16,20 +21,20 @@ COPYING distributed with the source files.
 #include <libintl.h>
 #endif
 
-Glib::ustring SettingsMessagesBase::get_caption(std::vector<Glib::ustring>::size_type index) {
+std::string SettingsMessagesBase::get_caption(std::vector<std::string>::size_type index) {
   if (index > captions.size()) {
     write_error("Yikes, the message index is too large in SettingsMessagesBase::get_caption.\n"
 		"Please report the bug to the author.\n");
-    return Glib::ustring(); // return empty string
+    return std::string(); // return empty string
   }
   else return captions[index];
 }
 
-Glib::ustring SettingsMessagesBase::get_message(std::vector<Glib::ustring>::size_type index) {
+std::string SettingsMessagesBase::get_message(std::vector<std::string>::size_type index) {
   if (index > messages.size()) {
     write_error("Yikes, the message index is too large in SettingsMessagesBase::get_message.\n"
 		"Please report the bug to the author.\n");
-    return Glib::ustring(); // return empty string
+    return std::string(); // return empty string
   }
   else return messages[index];
 }
@@ -209,60 +214,52 @@ PageMessages::PageMessages(void): SettingsMessagesBase(res + 1) {
   captions[res] = gettext("efax-gtk help: Sent Fax Resolution");
 }
 
-SettingsHelpDialog::SettingsHelpDialog(const int standard_size, const Glib::ustring& text,
-				       const Glib::ustring& caption, Gtk::Window& window):
-                                            Gtk::Window(Gtk::WINDOW_TOPLEVEL), in_exec_loop(false),
-					    close_button(Gtk::Stock::CLOSE),
-					    button_box(Gtk::BUTTONBOX_SPREAD),
-					    label(text), table(2, 1),
-					    parent(window) {
 
+namespace { // callback for internal use only
 
-  label.set_line_wrap(true);
-  button_box.add(close_button);
-
-  table.attach(label, 0, 1, 0, 1, Gtk::FILL | Gtk::EXPAND,
-	 Gtk::FILL | Gtk::EXPAND, standard_size/2, standard_size/4);
-
-  table.attach(button_box, 0, 1, 1, 2, Gtk::FILL | Gtk::EXPAND,
-	 Gtk::SHRINK, standard_size/2, standard_size/4);
-
-  add(table);
-
-  close_button.signal_clicked().connect(sigc::mem_fun(*this, &SettingsHelpDialog::kill_dialog));
-  close_button.set_flags(Gtk::CAN_DEFAULT);
-
-  set_title(caption);
-  set_transient_for(parent);
-  set_type_hint(Gdk::WINDOW_TYPE_HINT_DIALOG);
-  parent.set_sensitive(false);
-  set_modal(true);
-
-
-  set_border_width(standard_size/4);
-
-  set_position(Gtk::WIN_POS_CENTER_ON_PARENT);
-  set_resizable(false);
-
-  set_icon(prog_config.window_icon_r);
-
-  show_all();
+void SettingsHelpDialogCB::settings_help_button_clicked(GtkWidget*, void* data) {
+  static_cast<SettingsHelpDialog*>(data)->close();
 }
 
-void SettingsHelpDialog::exec(void) {
-  in_exec_loop = true;
-  Gtk::Main::run();
-}
+} // anonymous namespace
 
-void SettingsHelpDialog::kill_dialog(void) {
-  parent.set_sensitive(true);
-  hide_all();
-  if (in_exec_loop) Gtk::Main::quit();
-  // if we have not called exec(), then this dialog is self-owning and it is safe to call `delete this'
-  else delete this;
-}
 
-bool SettingsHelpDialog::on_delete_event(GdkEventAny*) {
-  kill_dialog();
-  return true; // returning true prevents destroy sig being emitted
+SettingsHelpDialog::SettingsHelpDialog(const int standard_size, const std::string& text,
+				       const std::string& caption, GtkWindow* parent_p):
+                                            WinBase(caption.c_str(),
+						    prog_config.window_icon_h,
+						    true, parent_p) {
+
+  close_button_p = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
+  GtkWidget* button_box_p = gtk_hbutton_box_new();
+  GtkWidget* label_p = gtk_label_new(text.c_str());
+  GtkTable* table_p = GTK_TABLE(gtk_table_new(2, 1, false));
+  
+  gtk_button_box_set_layout(GTK_BUTTON_BOX(button_box_p), GTK_BUTTONBOX_SPREAD);
+  gtk_container_add(GTK_CONTAINER(button_box_p), close_button_p);
+
+  gtk_label_set_line_wrap(GTK_LABEL(label_p), true);
+  
+  gtk_table_attach(table_p, label_p,
+		   0, 1, 0, 1,
+		   GtkAttachOptions(GTK_FILL | GTK_EXPAND), GtkAttachOptions(GTK_FILL | GTK_EXPAND),
+		   standard_size/2, standard_size/4);
+
+  gtk_table_attach(table_p, button_box_p,
+		   0, 1, 1, 2,
+		   GtkAttachOptions(GTK_FILL | GTK_EXPAND), GTK_SHRINK,
+		   standard_size/2, standard_size/4);
+  
+  g_signal_connect(G_OBJECT(close_button_p), "clicked",
+		   G_CALLBACK(SettingsHelpDialogCB::settings_help_button_clicked), this);
+
+  GTK_WIDGET_SET_FLAGS(close_button_p, GTK_CAN_DEFAULT);
+
+  gtk_container_set_border_width(GTK_CONTAINER(get_win()), standard_size/4);
+  gtk_container_add(GTK_CONTAINER(get_win()), GTK_WIDGET(table_p));
+  gtk_window_set_position(get_win(), GTK_WIN_POS_CENTER_ON_PARENT);
+  gtk_window_set_type_hint(get_win(), GDK_WINDOW_TYPE_HINT_DIALOG);
+  gtk_window_set_resizable(get_win(), false);
+
+  gtk_widget_show_all(GTK_WIDGET(get_win()));
 }
