@@ -1,4 +1,4 @@
-/* Copyright (C) 2003 2004 Chris Vine
+/* Copyright (C) 2003 to 2005 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -14,49 +14,50 @@ COPYING distributed with the source files.
 #include <string>
 #include <list>
 #include <utility>
+#include <memory>
 
 #include <netinet/in.h>
 
-#include <glibmm/thread.h>
-#include <glibmm/dispatcher.h>
-
 #include <sigc++/sigc++.h>
 
 #include "sem_sync.h"
 #include "shared_ptr.h"
+#include "thread.h"
+#include "mutex.h"
+#include "notifier.h"
 
 typedef std::list<std::pair<std::string, unsigned int> > FilenamesList;
 
 
-class Socket_server: public sigc::trackable {
+class SocketServer: public sigc::trackable {
 
   bool server_running;
   bool other_sock_client_address;
-  bool closing_socket;
   int serve_sock_fd;
   unsigned int count;
   std::string working_dir;
-  Glib::Mutex working_dir_mutex;
-
-  Sem_sync close_sem;
+  Thread::Mutex working_dir_mutex;
 
   // make filenames_mutex mutable so that we can use it in
-  // the const member function Socket_server::get_filenames()
-  // we can use a Shared_ptr rather than a Shared_lock_ptr
+  // the const member function SocketServer::get_filenames()
+  // we can use a SharedPtr rather than a Shared_lock_ptr
   // object here - filenames_mutex will do the locking of the
   // contained object and we do not actually copy filenames_s
   // in more than one thread (it is only copied in the main (GUI)
   // thread)
-  Shared_ptr<FilenamesList> filenames_s;
-  mutable Glib::Mutex filenames_mutex;
+  SharedPtr<FilenamesList> filenames_s;
+  mutable Thread::Mutex filenames_mutex;
 
-  Glib::Mutex socket_mutex;
-  Glib::Mutex stdout_mutex;
-  Glib::Mutex fax_to_send_mutex;
+  std::auto_ptr<Thread::Thread> socket_thread_h;
 
-  Glib::Dispatcher stdout_notify;
-  Glib::Cond stdout_cond;
-  Glib::Cond fax_to_send_cond;
+  Thread::Mutex stdout_mutex;
+  Thread::Mutex fax_to_send_mutex;
+
+  Notifier socket_error_notify;
+
+  Notifier stdout_notify;
+  Thread::Cond stdout_cond;
+  Thread::Cond fax_to_send_cond;
 
   std::string stdout_text;
   void write_stdout_from_thread(const char*);
@@ -72,28 +73,29 @@ class Socket_server: public sigc::trackable {
   void read_queued_faxes(void);
   void save_queued_faxes(void);
   bool is_valid_peer(const sockaddr_in&);
+  void cleanup(void);
 
   // this is a concrete class not to be copied or assigned
-  Socket_server(const Socket_server&);
-  void operator=(const Socket_server&);
+  SocketServer(const SocketServer&);
+  void operator=(const SocketServer&);
 public:
 
-  // Glib::Dispatcher is a thread safe signal
+  // Notifier is a thread safe signal
   // the main GUI thread should connect to it to be
   // notified when the server has received a fax which
   // is to be sent by efax-gtk or a fax has been
-  // removed from the list using Socket_server::remove_file()
-  Glib::Dispatcher filelist_changed_notify;
+  // removed from the list using SocketServer::remove_file()
+  Notifier filelist_changed_notify;
   // this signal will also be emitted if the reason for the
   // change was a fax received from the socket for sending
   // (the fax particulars can be obtained with get_fax_to_send())
-  Glib::Dispatcher fax_to_send_notify;
+  Notifier fax_to_send_notify;
 
   sigc::signal1<void, const char*> stdout_message;
 
   void add_file(const std::string&);
   int remove_file(const std::string&);
-  std::pair<Shared_ptr<FilenamesList>, Shared_ptr<Glib::Mutex::Lock> > get_filenames(void) const;
+  std::pair<SharedPtr<FilenamesList>, SharedPtr<Thread::Mutex::Lock> > get_filenames(void) const;
   std::pair<std::string, unsigned int> get_fax_to_send(void);  
   bool is_server_running(void) const {return server_running;}
   std::string get_port(void) const {return port;}
@@ -101,8 +103,8 @@ public:
   int get_count(void) const {return count;}
   void start(const std::string& port, bool other_sock_client_address);
   void stop(void);
-  Socket_server(void);
-  ~Socket_server(void) {if (server_running) stop();}
+  SocketServer(void);
+  ~SocketServer(void) {if (server_running) stop();}
 };
 
 #endif
